@@ -1,12 +1,12 @@
 """
 DynamoDB Models for PynamoDB
 """
-import json
 import random
 import time
 import logging
 import warnings
 import sys
+from copy import deepcopy
 from inspect import getmembers
 from typing import Any
 from typing import Dict
@@ -24,6 +24,7 @@ from typing import TypeVar
 from typing import Union
 from typing import cast
 
+from pynamodb._schema import ModelSchema
 from pynamodb.connection.base import MetaTable
 
 if sys.version_info >= (3, 8):
@@ -40,26 +41,22 @@ from pynamodb.attributes import (
 from pynamodb.connection.table import TableConnection
 from pynamodb.expressions.condition import Condition
 from pynamodb.types import HASH, RANGE
-from pynamodb.indexes import Index, GlobalSecondaryIndex, LocalSecondaryIndex
+from pynamodb.indexes import Index
 from pynamodb.pagination import ResultIterator
-from pynamodb.settings import get_settings_value, OperationSettings
+from pynamodb.settings import get_settings_value
 from pynamodb import constants
 from pynamodb.constants import (
-    ATTR_DEFINITIONS, ATTR_NAME, ATTR_TYPE, KEY_SCHEMA,
-    KEY_TYPE, ITEM, READ_CAPACITY_UNITS, WRITE_CAPACITY_UNITS,
-    RANGE_KEY, ATTRIBUTES, PUT, DELETE, RESPONSES,
-    INDEX_NAME, PROVISIONED_THROUGHPUT, PROJECTION, ALL_NEW,
-    GLOBAL_SECONDARY_INDEXES, LOCAL_SECONDARY_INDEXES, KEYS,
-    PROJECTION_TYPE, NON_KEY_ATTRIBUTES,
-    TABLE_STATUS, ACTIVE, RETURN_VALUES, BATCH_GET_PAGE_LIMIT,
+    ATTR_NAME, ATTR_TYPE,
+    KEY_TYPE, ITEM,
+    ATTRIBUTES, PUT, DELETE, RESPONSES,
+    ALL_NEW,
+    KEYS,
+    TABLE_STATUS, ACTIVE, BATCH_GET_PAGE_LIMIT,
     UNPROCESSED_KEYS, PUT_REQUEST, DELETE_REQUEST,
     BATCH_WRITE_PAGE_LIMIT,
     META_CLASS_NAME, REGION, HOST, NULL,
-    COUNT, ITEM_COUNT, KEY, UNPROCESSED_ITEMS, STREAM_VIEW_TYPE,
-    STREAM_SPECIFICATION, STREAM_ENABLED, BILLING_MODE, PAY_PER_REQUEST_BILLING_MODE, TAGS, TABLE_NAME
+    COUNT, ITEM_COUNT, KEY, UNPROCESSED_ITEMS,
 )
-from pynamodb.util import attribute_value_to_json
-from pynamodb.util import json_to_attribute_value
 
 _T = TypeVar('_T', bound='Model')
 _KeyType = Any
@@ -73,13 +70,12 @@ class BatchWrite(Generic[_T]):
     """
     A class for batch writes
     """
-    def __init__(self, model: Type[_T], auto_commit: bool = True, settings: OperationSettings = OperationSettings.default):
+    def __init__(self, model: Type[_T], auto_commit: bool = True):
         self.model = model
         self.auto_commit = auto_commit
         self.max_operations = BATCH_WRITE_PAGE_LIMIT
         self.pending_operations: List[Dict[str, Any]] = []
         self.failed_operations: List[Any] = []
-        self.settings = settings
 
     def save(self, put_item: _T) -> None:
         """
@@ -149,15 +145,15 @@ class BatchWrite(Generic[_T]):
         data = self.model._get_connection().batch_write_item(
             put_items=put_items,
             delete_items=delete_items,
-            settings=self.settings,
         )
         if data is None:
             return
         retries = 0
         unprocessed_items = data.get(UNPROCESSED_ITEMS, {}).get(self.model.Meta.table_name)
         while unprocessed_items:
-            sleep_time = random.randint(0, self.model.Meta.base_backoff_ms * (2 ** retries)) / 1000
-            time.sleep(sleep_time)
+            # TODO: we should consider using exponential backoff here
+            # TODO: it is somewhat unintuitive that we retry unprocessed items max_retry_attempts times,
+            # since each `batch_write_item` operation is also subject to max_retry_attempts
             retries += 1
             if retries >= self.model.Meta.max_retry_attempts:
                 self.failed_operations = unprocessed_items
@@ -169,12 +165,10 @@ class BatchWrite(Generic[_T]):
                     put_items.append(item.get(PUT_REQUEST).get(ITEM))  # type: ignore
                 elif DELETE_REQUEST in item:
                     delete_items.append(item.get(DELETE_REQUEST).get(KEY))  # type: ignore
-            log.info("Resending %d unprocessed keys for batch operation after %d seconds sleep",
-                     len(unprocessed_items), sleep_time)
+            log.info("Resending %d unprocessed keys for batch operation (retry %d)", len(unprocessed_items), retries)
             data = self.model._get_connection().batch_write_item(
                 put_items=put_items,
                 delete_items=delete_items,
-                settings=self.settings,
             )
             unprocessed_items = data.get(UNPROCESSED_ITEMS, {}).get(self.model.Meta.table_name)
 
@@ -187,7 +181,6 @@ class MetaProtocol(Protocol):
     host: Optional[str]
     connect_timeout_seconds: int
     read_timeout_seconds: int
-    base_backoff_ms: int
     max_retry_attempts: int
     max_pool_connections: int
     extra_headers: Mapping[str, str]
@@ -246,8 +239,6 @@ class MetaModel(AttributeContainerMeta):
                         setattr(attr_obj, 'connect_timeout_seconds', get_settings_value('connect_timeout_seconds'))
                     if not hasattr(attr_obj, 'read_timeout_seconds'):
                         setattr(attr_obj, 'read_timeout_seconds', get_settings_value('read_timeout_seconds'))
-                    if not hasattr(attr_obj, 'base_backoff_ms'):
-                        setattr(attr_obj, 'base_backoff_ms', get_settings_value('base_backoff_ms'))
                     if not hasattr(attr_obj, 'max_retry_attempts'):
                         setattr(attr_obj, 'max_retry_attempts', get_settings_value('max_retry_attempts'))
                     if not hasattr(attr_obj, 'max_pool_connections'):
@@ -277,6 +268,10 @@ class MetaModel(AttributeContainerMeta):
         """
         cls._indexes = {}
         for name, index in getmembers(cls, lambda o: isinstance(o, Index)):
+            # Store a local reference to the containing Model class on a copy of the index to support polymorphism.
+            index = deepcopy(index)
+            index._model = cls
+            setattr(cls, name, index)
             cls._indexes[index.Meta.index_name] = index
 
 
@@ -285,7 +280,7 @@ class Model(AttributeContainer, metaclass=MetaModel):
     Defines a `PynamoDB` Model
 
     This model is backed by a table in DynamoDB.
-    You can create the table by with the ``create_table`` method.
+    You can create the table with the ``create_table`` method.
     """
 
     # These attributes are named to avoid colliding with user defined
@@ -327,7 +322,6 @@ class Model(AttributeContainer, metaclass=MetaModel):
         items: Iterable[Union[_KeyType, Iterable[_KeyType]]],
         consistent_read: Optional[bool] = None,
         attributes_to_get: Optional[Sequence[str]] = None,
-        settings: OperationSettings = OperationSettings.default
     ) -> Iterator[_T]:
         """
         BatchGetItem for this model
@@ -346,7 +340,6 @@ class Model(AttributeContainer, metaclass=MetaModel):
                         keys_to_get,
                         consistent_read=consistent_read,
                         attributes_to_get=attributes_to_get,
-                        settings=settings,
                     )
                     for batch_item in page:
                         yield cls.from_raw_data(batch_item)
@@ -356,15 +349,23 @@ class Model(AttributeContainer, metaclass=MetaModel):
                         keys_to_get = []
             item = items.pop()
             if range_key_attribute:
-                hash_key, range_key = cls._serialize_keys(item[0], item[1])  # type: ignore
+                if isinstance(item, str):
+                    raise ValueError(f'Invalid key value {item!r}: '
+                                     'expected non-str iterable with exactly 2 elements (hash key, range key)')
+                try:
+                    hash_key, range_key = item
+                except (TypeError, ValueError):
+                    raise ValueError(f'Invalid key value {item!r}: '
+                                     'expected iterable with exactly 2 elements (hash key, range key)')
+                hash_key_ser, range_key_ser = cls._serialize_keys(hash_key, range_key)
                 keys_to_get.append({
-                    hash_key_attribute.attr_name: hash_key,
-                    range_key_attribute.attr_name: range_key
+                    hash_key_attribute.attr_name: hash_key_ser,
+                    range_key_attribute.attr_name: range_key_ser,
                 })
             else:
-                hash_key = cls._serialize_keys(item)[0]
+                hash_key_ser, _ = cls._serialize_keys(item)
                 keys_to_get.append({
-                    hash_key_attribute.attr_name: hash_key
+                    hash_key_attribute.attr_name: hash_key_ser
                 })
 
         while keys_to_get:
@@ -372,7 +373,6 @@ class Model(AttributeContainer, metaclass=MetaModel):
                 keys_to_get,
                 consistent_read=consistent_read,
                 attributes_to_get=attributes_to_get,
-                settings=settings,
             )
             for batch_item in page:
                 yield cls.from_raw_data(batch_item)
@@ -382,7 +382,7 @@ class Model(AttributeContainer, metaclass=MetaModel):
                 keys_to_get = []
 
     @classmethod
-    def batch_write(cls: Type[_T], auto_commit: bool = True, settings: OperationSettings = OperationSettings.default) -> BatchWrite[_T]:
+    def batch_write(cls: Type[_T], auto_commit: bool = True) -> BatchWrite[_T]:
         """
         Returns a BatchWrite context manager for a batch operation.
 
@@ -392,36 +392,36 @@ class Model(AttributeContainer, metaclass=MetaModel):
                             passed here, changes automatically commit on context exit
                             (whether successful or not).
         """
-        return BatchWrite(cls, auto_commit=auto_commit, settings=settings)
+        return BatchWrite(cls, auto_commit=auto_commit)
 
-    def __repr__(self) -> str:
-        hash_key, range_key = self._get_serialized_keys()
-        if self._range_keyname:
-            msg = "{}<{}, {}>".format(self.Meta.table_name, hash_key, range_key)
-        else:
-            msg = "{}<{}>".format(self.Meta.table_name, hash_key)
-        return msg
-
-    def delete(self, condition: Optional[Condition] = None, settings: OperationSettings = OperationSettings.default) -> Any:
+    def delete(self, condition: Optional[Condition] = None, *, add_version_condition: bool = True) -> Any:
         """
-        Deletes this object from dynamodb
+        Deletes this object from DynamoDB.
 
+        :param add_version_condition: For models which have a :class:`~pynamodb.attributes.VersionAttribute`,
+          specifies whether the item should only be deleted if its current version matches the expected one.
+          Set to `False` for a 'delete anyway' strategy.
         :raises pynamodb.exceptions.DeleteError: If the record can not be deleted
         """
         hk_value, rk_value = self._get_hash_range_key_serialized_values()
+
         version_condition = self._handle_version_attribute()
-        if version_condition is not None:
+        if add_version_condition and version_condition is not None:
             condition &= version_condition
 
-        return self._get_connection().delete_item(hk_value, range_key=rk_value, condition=condition, settings=settings)
+        return self._get_connection().delete_item(hk_value, range_key=rk_value, condition=condition)
 
-    def update(self, actions: List[Action], condition: Optional[Condition] = None, settings: OperationSettings = OperationSettings.default) -> Any:
+    def update(self, actions: List[Action], condition: Optional[Condition] = None, *, add_version_condition: bool = True) -> Any:
         """
         Updates an item using the UpdateItem operation.
 
         :param actions: a list of Action updates to apply
         :param condition: an optional Condition on which to update
         :param settings: per-operation settings
+        :param add_version_condition: For models which have a :class:`~pynamodb.attributes.VersionAttribute`,
+          specifies whether only to update if the version matches the model that is currently loaded.
+          Set to `False` for a 'last write wins' strategy.
+          Regardless, the version will always be incremented to prevent "rollbacks" by concurrent :meth:`save` calls.
         :raises ModelInstance.DoesNotExist: if the object to be updated does not exist
         :raises pynamodb.exceptions.UpdateError: if the `condition` is not met
         """
@@ -430,10 +430,10 @@ class Model(AttributeContainer, metaclass=MetaModel):
 
         hk_value, rk_value = self._get_hash_range_key_serialized_values()
         version_condition = self._handle_version_attribute(actions=actions)
-        if version_condition is not None:
+        if add_version_condition and version_condition is not None:
             condition &= version_condition
 
-        data = self._get_connection().update_item(hk_value, range_key=rk_value, return_values=ALL_NEW, condition=condition, actions=actions, settings=settings)
+        data = self._get_connection().update_item(hk_value, range_key=rk_value, return_values=ALL_NEW, condition=condition, actions=actions)
         item_data = data[ATTRIBUTES]
         stored_cls = self._get_discriminator_class(item_data)
         if stored_cls and stored_cls != type(self):
@@ -441,17 +441,16 @@ class Model(AttributeContainer, metaclass=MetaModel):
         self.deserialize(item_data)
         return data
 
-    def save(self, condition: Optional[Condition] = None, settings: OperationSettings = OperationSettings.default) -> Dict[str, Any]:
+    def save(self, condition: Optional[Condition] = None, *, add_version_condition: bool = True) -> Dict[str, Any]:
         """
         Save this object to dynamodb
         """
-        args, kwargs = self._get_save_args(condition=condition)
-        kwargs['settings'] = settings
+        args, kwargs = self._get_save_args(condition=condition, add_version_condition=add_version_condition)
         data = self._get_connection().put_item(*args, **kwargs)
         self.update_local_version_attribute()
         return data
 
-    def refresh(self, consistent_read: bool = False, settings: OperationSettings = OperationSettings.default) -> None:
+    def refresh(self, consistent_read: bool = False) -> None:
         """
         Retrieves this object's data from dynamodb and syncs this local object
 
@@ -460,7 +459,7 @@ class Model(AttributeContainer, metaclass=MetaModel):
         :raises ModelInstance.DoesNotExist: if the object to be updated does not exist
         """
         hk_value, rk_value = self._get_hash_range_key_serialized_values()
-        attrs = self._get_connection().get_item(hk_value, range_key=rk_value, consistent_read=consistent_read, settings=settings)
+        attrs = self._get_connection().get_item(hk_value, range_key=rk_value, consistent_read=consistent_read)
         item_data = attrs.get(ITEM, None)
         if item_data is None:
             raise self.DoesNotExist("This item does not exist in the table.")
@@ -474,11 +473,13 @@ class Model(AttributeContainer, metaclass=MetaModel):
         actions: List[Action],
         condition: Optional[Condition] = None,
         return_values_on_condition_failure: Optional[str] = None,
+        *,
+        add_version_condition: bool = True,
     ) -> Dict[str, Any]:
         hk_value, rk_value = self._get_hash_range_key_serialized_values()
 
         version_condition = self._handle_version_attribute(actions=actions)
-        if version_condition is not None:
+        if add_version_condition and version_condition is not None:
             condition &= version_condition
 
         return self._get_connection().get_operation_kwargs(hk_value, range_key=rk_value, key=KEY, actions=actions, condition=condition, return_values_on_condition_failure=return_values_on_condition_failure)
@@ -487,11 +488,13 @@ class Model(AttributeContainer, metaclass=MetaModel):
         self,
         condition: Optional[Condition] = None,
         return_values_on_condition_failure: Optional[str] = None,
+        *,
+        add_version_condition: bool = True,
     ) -> Dict[str, Any]:
         hk_value, rk_value = self._get_hash_range_key_serialized_values()
 
         version_condition = self._handle_version_attribute()
-        if version_condition is not None:
+        if add_version_condition and version_condition is not None:
             condition &= version_condition
 
         return self._get_connection().get_operation_kwargs(hk_value, range_key=rk_value, key=KEY, condition=condition, return_values_on_condition_failure=return_values_on_condition_failure)
@@ -501,7 +504,7 @@ class Model(AttributeContainer, metaclass=MetaModel):
         condition: Optional[Condition] = None,
         return_values_on_condition_failure: Optional[str] = None,
     ) -> Dict[str, Any]:
-        args, save_kwargs = self._get_save_args(null_check=True, condition=condition)
+        args, save_kwargs = self._get_save_args(condition=condition)
         save_kwargs['key'] = ITEM
         save_kwargs['return_values_on_condition_failure'] = return_values_on_condition_failure
         return self._get_connection().get_operation_kwargs(*args, **save_kwargs)
@@ -527,7 +530,6 @@ class Model(AttributeContainer, metaclass=MetaModel):
         range_key: Optional[_KeyType] = None,
         consistent_read: bool = False,
         attributes_to_get: Optional[Sequence[Text]] = None,
-        settings: OperationSettings = OperationSettings.default
     ) -> _T:
         """
         Returns a single object using the provided keys
@@ -545,7 +547,6 @@ class Model(AttributeContainer, metaclass=MetaModel):
             range_key=range_key,
             consistent_read=consistent_read,
             attributes_to_get=attributes_to_get,
-            settings=settings,
         )
         if data:
             item_data = data.get(ITEM)
@@ -576,7 +577,6 @@ class Model(AttributeContainer, metaclass=MetaModel):
         index_name: Optional[str] = None,
         limit: Optional[int] = None,
         rate_limit: Optional[float] = None,
-        settings: OperationSettings = OperationSettings.default,
     ) -> int:
         """
         Provides a filtered count
@@ -619,7 +619,6 @@ class Model(AttributeContainer, metaclass=MetaModel):
             query_kwargs,
             limit=limit,
             rate_limit=rate_limit,
-            settings=settings,
         )
 
         # iterate through results
@@ -641,7 +640,6 @@ class Model(AttributeContainer, metaclass=MetaModel):
         attributes_to_get: Optional[Iterable[str]] = None,
         page_size: Optional[int] = None,
         rate_limit: Optional[float] = None,
-        settings: OperationSettings = OperationSettings.default,
     ) -> ResultIterator[_T]:
         """
         Provides a high level query API
@@ -691,7 +689,6 @@ class Model(AttributeContainer, metaclass=MetaModel):
             map_fn=cls.from_raw_data,
             limit=limit,
             rate_limit=rate_limit,
-            settings=settings,
         )
 
     @classmethod
@@ -707,7 +704,6 @@ class Model(AttributeContainer, metaclass=MetaModel):
         index_name: Optional[str] = None,
         rate_limit: Optional[float] = None,
         attributes_to_get: Optional[Sequence[str]] = None,
-        settings: OperationSettings = OperationSettings.default,
     ) -> ResultIterator[_T]:
         """
         Iterates through all items in the table
@@ -750,7 +746,6 @@ class Model(AttributeContainer, metaclass=MetaModel):
             map_fn=cls.from_raw_data,
             limit=limit,
             rate_limit=rate_limit,
-            settings=settings,
         )
 
     @classmethod
@@ -797,27 +792,33 @@ class Model(AttributeContainer, metaclass=MetaModel):
         """
         if not cls.exists():
             schema = cls._get_schema()
+            operation_kwargs: Dict[str, Any] = {
+                'attribute_definitions': schema['attribute_definitions'],
+                'key_schema': schema['key_schema'],
+                'global_secondary_indexes': schema['global_secondary_indexes'],
+                'local_secondary_indexes': schema['local_secondary_indexes'],
+            }
             if hasattr(cls.Meta, 'read_capacity_units'):
-                schema['read_capacity_units'] = cls.Meta.read_capacity_units
+                operation_kwargs['read_capacity_units'] = cls.Meta.read_capacity_units
             if hasattr(cls.Meta, 'write_capacity_units'):
-                schema['write_capacity_units'] = cls.Meta.write_capacity_units
+                operation_kwargs['write_capacity_units'] = cls.Meta.write_capacity_units
             if hasattr(cls.Meta, 'stream_view_type'):
-                schema['stream_specification'] = {
+                operation_kwargs['stream_specification'] = {
                     'stream_enabled': True,
                     'stream_view_type': cls.Meta.stream_view_type
                 }
             if hasattr(cls.Meta, 'billing_mode'):
-                schema['billing_mode'] = cls.Meta.billing_mode
+                operation_kwargs['billing_mode'] = cls.Meta.billing_mode
             if hasattr(cls.Meta, 'tags'):
-                schema['tags'] = cls.Meta.tags
+                operation_kwargs['tags'] = cls.Meta.tags
             if read_capacity_units is not None:
-                schema['read_capacity_units'] = read_capacity_units
+                operation_kwargs['read_capacity_units'] = read_capacity_units
             if write_capacity_units is not None:
-                schema['write_capacity_units'] = write_capacity_units
+                operation_kwargs['write_capacity_units'] = write_capacity_units
             if billing_mode is not None:
-                schema['billing_mode'] = billing_mode
+                operation_kwargs['billing_mode'] = billing_mode
             cls._get_connection().create_table(
-                **schema
+                **operation_kwargs
             )
         if wait:
             while True:
@@ -853,11 +854,12 @@ class Model(AttributeContainer, metaclass=MetaModel):
 
     # Private API below
     @classmethod
-    def _get_schema(cls) -> Dict[str, Any]:
+    def _get_schema(cls) -> ModelSchema:
         """
         Returns the schema for this table
         """
-        schema: Dict[str, List] = {
+
+        schema: ModelSchema = {
             'attribute_definitions': [],
             'key_schema': [],
             'global_secondary_indexes': [],
@@ -869,47 +871,35 @@ class Model(AttributeContainer, metaclass=MetaModel):
                     ATTR_NAME: attr_cls.attr_name,
                     ATTR_TYPE: attr_cls.attr_type
                 })
-            if attr_cls.is_hash_key:
                 schema['key_schema'].append({
-                    KEY_TYPE: HASH,
+                    KEY_TYPE: HASH if attr_cls.is_hash_key else RANGE,
                     ATTR_NAME: attr_cls.attr_name
                 })
-            elif attr_cls.is_range_key:
-                schema['key_schema'].append({
-                    KEY_TYPE: RANGE,
-                    ATTR_NAME: attr_cls.attr_name
-                })
-        for index in cls._indexes.values():
-            index_schema = index._get_schema()
-            if isinstance(index, GlobalSecondaryIndex):
-                if getattr(cls.Meta, 'billing_mode', None) == PAY_PER_REQUEST_BILLING_MODE:
-                    index_schema.pop('provisioned_throughput', None)
-                schema['global_secondary_indexes'].append(index_schema)
-            else:
-                schema['local_secondary_indexes'].append(index_schema)
-        attr_names = {key_schema[ATTR_NAME]
-                      for index_schema in (*schema['global_secondary_indexes'], *schema['local_secondary_indexes'])
-                      for key_schema in index_schema['key_schema']}
-        attr_keys = {attr[ATTR_NAME] for attr in schema['attribute_definitions']}
-        for attr_name in attr_names:
-            if attr_name not in attr_keys:
-                attr_cls = cls.get_attributes()[cls._dynamo_to_python_attr(attr_name)]
-                schema['attribute_definitions'].append({
-                    ATTR_NAME: attr_cls.attr_name,
-                    ATTR_TYPE: attr_cls.attr_type
-                })
+
+        indexes = cls._indexes.copy()
+        # add indexes from derived classes that we might initialize
+        discriminator_attr = cls._get_discriminator_attribute()
+        if discriminator_attr is not None:
+            for model_cls in discriminator_attr.get_registered_subclasses(Model):
+                indexes.update(model_cls._indexes)
+
+        for index in indexes.values():
+            index._update_model_schema(schema)
+
         return schema
 
-    def _get_save_args(self, null_check: bool = True, condition: Optional[Condition] = None) -> Tuple[Iterable[Any], Dict[str, Any]]:
+    def _get_save_args(self, condition: Optional[Condition] = None, *, add_version_condition: bool = True) -> Tuple[Iterable[Any], Dict[str, Any]]:
         """
         Gets the proper *args, **kwargs for saving and retrieving this object
 
-        This is used for serializing items to be saved, or for serializing just the keys.
+        This is used for serializing items to be saved.
 
-        :param null_check: If True, then attributes are checked for null.
         :param condition: If set, a condition
+        :param add_version_condition: For models which have a :class:`~pynamodb.attributes.VersionAttribute`,
+          specifies whether the item should only be saved if its current version matches the expected one.
+          Set to `False` for a 'last-write-wins' strategy.
         """
-        attribute_values = self.serialize(null_check)
+        attribute_values = self.serialize(null_check=True)
         hash_key_attribute = self._hash_key_attribute()
         hash_key = attribute_values.pop(hash_key_attribute.attr_name, {}).get(hash_key_attribute.attr_type)
         range_key = None
@@ -921,7 +911,7 @@ class Model(AttributeContainer, metaclass=MetaModel):
         if range_key is not None:
             kwargs['range_key'] = range_key
         version_condition = self._handle_version_attribute(attributes=attribute_values)
-        if version_condition is not None:
+        if add_version_condition and version_condition is not None:
             condition &= version_condition
         kwargs['attributes'] = attribute_values
         kwargs['condition'] = condition
@@ -1019,7 +1009,7 @@ class Model(AttributeContainer, metaclass=MetaModel):
         return self._serialize_keys(hash_key, range_key)
 
     @classmethod
-    def _batch_get_page(cls, keys_to_get, consistent_read, attributes_to_get, settings: OperationSettings):
+    def _batch_get_page(cls, keys_to_get, consistent_read, attributes_to_get):
         """
         Returns a single page from BatchGetItem
         Also returns any unprocessed items
@@ -1030,7 +1020,7 @@ class Model(AttributeContainer, metaclass=MetaModel):
         """
         log.debug("Fetching a BatchGetItem page")
         data = cls._get_connection().batch_get_item(
-            keys_to_get, consistent_read=consistent_read, attributes_to_get=attributes_to_get, settings=settings,
+            keys_to_get, consistent_read=consistent_read, attributes_to_get=attributes_to_get,
         )
         item_data = data.get(RESPONSES).get(cls.Meta.table_name)  # type: ignore
         unprocessed_items = data.get(UNPROCESSED_KEYS).get(cls.Meta.table_name, {}).get(KEYS, None)  # type: ignore
@@ -1087,7 +1077,6 @@ class Model(AttributeContainer, metaclass=MetaModel):
                                               connect_timeout_seconds=cls.Meta.connect_timeout_seconds,
                                               read_timeout_seconds=cls.Meta.read_timeout_seconds,
                                               max_retry_attempts=cls.Meta.max_retry_attempts,
-                                              base_backoff_ms=cls.Meta.base_backoff_ms,
                                               max_pool_connections=cls.Meta.max_pool_connections,
                                               extra_headers=cls.Meta.extra_headers,
                                               aws_access_key_id=cls.Meta.aws_access_key_id,
@@ -1128,24 +1117,23 @@ class Model(AttributeContainer, metaclass=MetaModel):
 
     def serialize(self, null_check: bool = True) -> Dict[str, Dict[str, Any]]:
         """
-        Serialize attribute values for DynamoDB
+        Serializes a model for botocore's DynamoDB client.
+
+        .. warning::
+            BINARY and BINARY_SET attributes (whether top-level or nested) serialization would contain
+            :code:`bytes` objects which are not JSON-serializable by the :code:`json` module.
+
+            Use :meth:`~pynamodb.attributes.AttributeContainer.to_dynamodb_dict`
+            and :meth:`~pynamodb.attributes.AttributeContainer.to_simple_dict` for JSON-serializable mappings.
         """
         return self._container_serialize(null_check=null_check)
 
     def deserialize(self, attribute_values: Dict[str, Dict[str, Any]]) -> None:
         """
-        Sets attributes sent back from DynamoDB on this object
+        Deserializes a model from botocore's DynamoDB client.
         """
         return self._container_deserialize(attribute_values=attribute_values)
 
-    def to_json(self) -> str:
-        return json.dumps({k: attribute_value_to_json(v) for k, v in self.serialize().items()})
-
-    def from_json(self, s: str) -> None:
-        attribute_values = {k: json_to_attribute_value(v) for k, v in json.loads(s).items()}
-        self._update_attribute_types(attribute_values)
-        self.deserialize(attribute_values)
-
 
 class _ModelFuture(Generic[_T]):
     """
